package com.tzld.piaoquan.api.controller.contentplatform;

import com.tzld.piaoquan.api.annotation.JwtIgnore;
import com.tzld.piaoquan.api.job.ContentPlatformVideoJob;
import com.tzld.piaoquan.api.model.param.IdParam;
import com.tzld.piaoquan.api.model.param.contentplatform.*;
import com.tzld.piaoquan.api.model.vo.contentplatform.GzhPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.QwPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.VideoContentItemVO;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformPlanService;
import com.tzld.piaoquan.growth.common.common.base.CommonResponse;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/contentPlatform/plan")
@CrossOrigin(origins = "*")
public class ContentPlatformPlanController {

    @Autowired
    ContentPlatformPlanService planService;

    @Autowired
    private ContentPlatformVideoJob videoJob;

    @ApiOperation(value = "公众号计划列表")
    @PostMapping("/gzh/list")
    public CommonResponse<Page<GzhPlanItemVO>> gzhPlanList(@RequestBody GzhPlanListParam param) {
        return CommonResponse.success(planService.gzhPlanList(param));
    }

    @ApiOperation(value = "公众号计划 创建/更新")
    @PostMapping("/gzh/save")
    public CommonResponse<Void> gzhPlanSave(@RequestBody GzhPlanSaveParam param) {
        planService.gzhPlanSave(param);
        return CommonResponse.success();
    }

    @ApiOperation(value = "公众号计划 删除")
    @PostMapping("/gzh/delete")
    public CommonResponse<Void> gzhPlanDelete(@RequestBody IdParam<Long> param) {
        planService.gzhPlanDelete(param.getId());
        return CommonResponse.success();
    }

    @ApiOperation(value = "公众号计划 平台发布开关修改")
    @PostMapping("/gzh/updateStagePublishStatus")
    public CommonResponse<Void> gzhUpdateStagePublishStatus(@RequestBody GzhPlanStagePublishStatusUpdateParam param) {
        planService.gzhUpdateStagePublishStatus(param.getId(), param.getStagePublishStatus());
        return CommonResponse.success();
    }

    @ApiOperation(value = "发布内容视频列表")
    @PostMapping("/videoContentList")
    public CommonResponse<Page<VideoContentItemVO>> getVideoContentList(@RequestBody VideoContentListParam param) {
        return CommonResponse.success(planService.getVideoContentList(param));
    }

    @ApiOperation(value = "发布内容视频品类列表")
    @GetMapping("/videoContentCategoryList")
    public CommonResponse<List<String>> getVideoContentCategoryList() {
        return CommonResponse.success(planService.getVideoContentCategoryList());
    }

    @ApiOperation(value = "发布内容视频封面截帧列表")
    @PostMapping("/videoContentCoverFrameList")
    public CommonResponse<List<String>> getVideoContentCoverFrameList(@RequestBody VideoContentCoverFrameParam param) {
        return CommonResponse.success(planService.getVideoContentCoverFrameList(param));
    }

    @ApiOperation(value = "企微计划列表")
    @PostMapping("/qw/list")
    public CommonResponse<Page<QwPlanItemVO>> qwPlanList(@RequestBody QwPlanListParam param) {
        return CommonResponse.success(planService.qwPlanList(param));
    }

    @ApiOperation(value = "企微计划 创建/更新")
    @PostMapping("/qw/save")
    public CommonResponse<List<QwPlanItemVO>> qwPlanSave(@RequestBody QwPlanSaveParam param) {
        return CommonResponse.success(planService.qwPlanSave(param));
    }

    @ApiOperation(value = "企微计划 删除")
    @PostMapping("/qw/delete")
    public CommonResponse<Void> qwPlanDelete(@RequestBody IdParam<Long> param) {
        planService.qwPlanDelete(param.getId());
        return CommonResponse.success();
    }

    @ApiOperation(value = "获取分享二维码")
    @GetMapping("/qw/getSharePic")
    public CommonResponse<String> getSharePic(@RequestParam String pageUrl) {
        return CommonResponse.success(planService.getSharePic(pageUrl));
    }

    @ApiOperation(value = "同步最近top1000视频", hidden = true)
    @JwtIgnore
    @GetMapping("/job/syncContentPlatformVideoJob")
    public CommonResponse<Void> syncContentPlatformVideoJob(String dateStr) {
        videoJob.syncContentPlatformVideoJob(dateStr);
        return CommonResponse.success();
    }

    @ApiOperation(value = "检查视频状态", hidden = true)
    @JwtIgnore
    @GetMapping("/job/checkContentPlatformVideoStatusJob")
    public CommonResponse<Void> checkContentPlatformVideoStatusJob(String dateStr) {
        videoJob.checkContentPlatformVideoStatusJob(dateStr);
        return CommonResponse.success();
    }

    @ApiOperation(value = "同步最近top1000视频统计数据", hidden = true)
    @JwtIgnore
    @GetMapping("/job/syncContentPlatformVideoDatastatJob")
    public CommonResponse<Void> syncContentPlatformVideoDatastatJob(String dateStr) {
        videoJob.syncContentPlatformVideoDatastatJob(dateStr);
        return CommonResponse.success();
    }
}
